@@ -4,17 +4,35 @@ Implements the Distutils 'install' command."""
 
 # created 1999/03/13, Greg Ward
 
-__revision__ = "$Id: install.py,v 1.54.2.1 2001/03/31 15:03:06 moshez Exp $"
+__revision__ = "$Id: install.py,v 1.60.6.2 2002/03/25 13:15:28 mwh Exp $"
 
 import sys, os, string
 from types import *
 from distutils.core import Command, DEBUG
 from distutils.sysconfig import get_config_vars
+from distutils.errors import DistutilsPlatformError
 from distutils.file_util import write_file
 from distutils.util import convert_path, subst_vars, change_root
 from distutils.errors import DistutilsOptionError
 from glob import glob
 
+if sys.version < "2.2":
+    WINDOWS_SCHEME = {
+        'purelib': '$base',
+        'platlib': '$base',
+        'headers': '$base/Include/$dist_name',
+        'scripts': '$base/Scripts',
+        'data'   : '$base',
+    }
+else:
+    WINDOWS_SCHEME = {
+        'purelib': '$base/Lib/site-packages',
+        'platlib': '$base/Lib/site-packages',
+        'headers': '$base/Include/$dist_name',
+        'scripts': '$base/Scripts',
+        'data'   : '$base',
+    }
+
 INSTALL_SCHEMES = {
     'unix_prefix': {
         'purelib': '$base/lib/python$py_version_short/site-packages',
@@ -30,13 +48,7 @@ INSTALL_SCHEMES = {
         'scripts': '$base/bin',
         'data'   : '$base',
         },
-    'nt': {
-        'purelib': '$base',
-        'platlib': '$base',
-        'headers': '$base/Include/$dist_name',
-        'scripts': '$base/Scripts',
-        'data'   : '$base',
-        },
+    'nt': WINDOWS_SCHEME,
     'mac': {
         'purelib': '$base/Lib/site-packages',
         'platlib': '$base/Lib/site-packages',
@@ -98,7 +110,7 @@ class install (Command):
         ('optimize=', 'O',
          "also compile with optimization: -O1 for \"python -O\", "
          "-O2 for \"python -OO\", and -O0 to disable [default: -O0]"),
-         
+
         # Miscellaneous control options
         ('force', 'f',
          "force installation (overwrite any existing files)"),
@@ -115,7 +127,7 @@ class install (Command):
          "filename in which to record list of installed files"),
         ]
 
-    boolean_options = ['force', 'skip-build']
+    boolean_options = ['compile', 'force', 'skip-build']
     negative_opt = {'no-compile' : 'compile'}
 
 
@@ -292,7 +304,7 @@ class install (Command):
                 self.install_lib = self.install_platlib
             else:
                 self.install_lib = self.install_purelib
-                    
+
 
         # Convert directories from Unix /-separated syntax to the local
         # convention.
@@ -340,7 +352,7 @@ class install (Command):
 
 
     def finalize_unix (self):
-        
+
         if self.install_base is not None or self.install_platbase is not None:
             if ((self.install_lib is None and
                  self.install_purelib is None and
@@ -413,7 +425,7 @@ class install (Command):
     def expand_basedirs (self):
         self._expand_attrs(['install_base',
                             'install_platbase',
-                            'root'])        
+                            'root'])
 
     def expand_dirs (self):
         self._expand_attrs(['install_purelib',
@@ -524,8 +536,7 @@ class install (Command):
     # -- Reporting methods ---------------------------------------------
 
     def get_outputs (self):
-        # This command doesn't have any outputs of its own, so just
-        # get the outputs of all its sub-commands.
+        # Assemble the outputs of all the sub-commands.
         outputs = []
         for cmd_name in self.get_sub_commands():
             cmd = self.get_finalized_command(cmd_name)
@@ -535,6 +546,10 @@ class install (Command):
                 if filename not in outputs:
                     outputs.append(filename)
 
+        if self.path_file and self.install_path_file:
+            outputs.append(os.path.join(self.install_libbase,
+                                        self.path_file + ".pth"))
+
         return outputs
 
     def get_inputs (self):
