@@ -23,14 +23,17 @@ Misc variables:
 
 """
 
-__version__ = "$Revision: 1.39.2.1 $"       # Code version
+__version__ = "$Revision: 1.56.4.1 $"       # Code version
 
 from types import *
 from copy_reg import dispatch_table, safe_constructors
-import string
 import marshal
 import sys
 import struct
+import re
+
+__all__ = ["PickleError", "PicklingError", "UnpicklingError", "Pickler",
+           "Unpickler", "dump", "dumps", "load", "loads"]
 
 format_version = "1.3"                     # File format version we write
 compatible_formats = ["1.0", "1.1", "1.2"] # Old format versions we can read
@@ -42,11 +45,21 @@ class PickleError(Exception): pass
 class PicklingError(PickleError): pass
 class UnpicklingError(PickleError): pass
 
+class _Stop(Exception):
+    def __init__(self, value):
+        self.value = value
+
 try:
     from org.python.core import PyStringMap
 except ImportError:
     PyStringMap = None
 
+try:
+    UnicodeType
+except NameError:
+    UnicodeType = None
+
+
 MARK            = '('
 STOP            = '.'
 POP             = '0'
@@ -89,6 +102,8 @@ EMPTY_TUPLE     = ')'
 SETITEMS        = 'u'
 BINFLOAT        = 'G'
 
+__all__.extend([x for x in dir() if re.match("[A-Z][A-Z0-9_]+$",x)])
+
 class Pickler:
 
     def __init__(self, file, bin = 0):
@@ -101,9 +116,9 @@ class Pickler:
         self.write(STOP)
 
     def put(self, i):
-        if (self.bin):
+        if self.bin:
             s = mdumps(i)[1:]
-            if (i < 256):
+            if i < 256:
                 return BINPUT + s[0]
 
             return LONG_BINPUT + s
@@ -111,31 +126,31 @@ class Pickler:
         return PUT + `i` + '\n'
 
     def get(self, i):
-        if (self.bin):
+        if self.bin:
             s = mdumps(i)[1:]
 
-            if (i < 256):
+            if i < 256:
                 return BINGET + s[0]
 
             return LONG_BINGET + s
 
         return GET + `i` + '\n'
-        
+
     def save(self, object, pers_save = 0):
         memo = self.memo
 
-        if (not pers_save):
+        if not pers_save:
             pid = self.persistent_id(object)
-            if (pid is not None):
+            if pid is not None:
                 self.save_pers(pid)
                 return
 
         d = id(object)
- 
+
         t = type(object)
 
-        if ((t is TupleType) and (len(object) == 0)):
-            if (self.bin):
+        if (t is TupleType) and (len(object) == 0):
+            if self.bin:
                 self.save_empty_tuple(object)
             else:
                 self.save_tuple(object)
@@ -148,6 +163,14 @@ class Pickler:
         try:
             f = self.dispatch[t]
         except KeyError:
+            try:
+                issc = issubclass(t, TypeType)
+            except TypeError: # t is not a class
+                issc = 0
+            if issc:
+                self.save_global(object)
+                return
+
             pid = self.inst_persistent_id(object)
             if pid is not None:
                 self.save_pers(pid)
@@ -171,20 +194,20 @@ class Pickler:
                 self.save_global(object, tup)
                 return
 
-            if (type(tup) is not TupleType):
+            if type(tup) is not TupleType:
                 raise PicklingError, "Value returned by %s must be a " \
                                      "tuple" % reduce
 
             l = len(tup)
-   
-            if ((l != 2) and (l != 3)):
+
+            if (l != 2) and (l != 3):
                 raise PicklingError, "tuple returned by %s must contain " \
                                      "only two or three elements" % reduce
 
             callable = tup[0]
             arg_tup  = tup[1]
-          
-            if (l > 2):
+
+            if l > 2:
                 state = tup[2]
             else:
                 state = None
@@ -193,7 +216,7 @@ class Pickler:
                 raise PicklingError, "Second element of tuple returned " \
                                      "by %s must be a tuple" % reduce
 
-            self.save_reduce(callable, arg_tup, state) 
+            self.save_reduce(callable, arg_tup, state)
             memo_len = len(memo)
             self.write(self.put(memo_len))
             memo[d] = (memo_len, object)
@@ -208,7 +231,7 @@ class Pickler:
         return None
 
     def save_pers(self, pid):
-        if (not self.bin):
+        if not self.bin:
             self.write(PERSID + str(pid) + '\n')
         else:
             self.save(pid, 1)
@@ -221,8 +244,8 @@ class Pickler:
         save(callable)
         save(arg_tup)
         write(REDUCE)
-        
-        if (state is not None):
+
+        if state is not None:
             save(state)
             write(BUILD)
 
@@ -233,19 +256,26 @@ class Pickler:
     dispatch[NoneType] = save_none
 
     def save_int(self, object):
-        if (self.bin):
-            i = mdumps(object)[1:]
-            if (i[-2:] == '\000\000'):
-                if (i[-3] == '\000'):
-                    self.write(BININT1 + i[:-3])
-                    return
-
-                self.write(BININT2 + i[:-2])
+        if self.bin:
+            # If the int is small enough to fit in a signed 4-byte 2's-comp
+            # format, we can store it more efficiently than the general
+            # case.
+            high_bits = object >> 31  # note that Python shift sign-extends
+            if  high_bits == 0 or high_bits == -1:
+                # All high bits are copies of bit 2**31, so the value
+                # fits in a 4-byte signed int.
+                i = mdumps(object)[1:]
+                assert len(i) == 4
+                if i[-2:] == '\000\000':    # fits in 2-byte unsigned int
+                    if i[-3] == '\000':     # fits in 1-byte unsigned int
+                        self.write(BININT1 + i[0])
+                    else:
+                        self.write(BININT2 + i[:2])
+                else:
+                    self.write(BININT + i)
                 return
-
-            self.write(BININT + i)
-        else:
-            self.write(INT + `object` + '\n')
+        # Text pickle, or int too big to fit in signed 4-byte format.
+        self.write(INT + `object` + '\n')
     dispatch[IntType] = save_int
 
     def save_long(self, object):
@@ -263,10 +293,10 @@ class Pickler:
         d = id(object)
         memo = self.memo
 
-        if (self.bin):
+        if self.bin:
             l = len(object)
             s = mdumps(l)[1:]
-            if (l < 256):
+            if l < 256:
                 self.write(SHORT_BINSTRING + s[0] + object)
             else:
                 self.write(BINSTRING + s + object)
@@ -282,14 +312,14 @@ class Pickler:
         d = id(object)
         memo = self.memo
 
-        if (self.bin):
+        if self.bin:
             encoding = object.encode('utf-8')
             l = len(encoding)
             s = mdumps(l)[1:]
             self.write(BINUNICODE + s + encoding)
         else:
-            object = object.replace(u"\\", u"\\u005c")
-            object = object.replace(u"\n", u"\\u000a")
+            object = object.replace("\\", "\\u005c")
+            object = object.replace("\n", "\\u000a")
             self.write(UNICODE + object.encode('raw-unicode-escape') + '\n')
 
         memo_len = len(memo)
@@ -297,6 +327,39 @@ class Pickler:
         memo[d] = (memo_len, object)
     dispatch[UnicodeType] = save_unicode
 
+    if StringType == UnicodeType:
+        # This is true for Jython
+        def save_string(self, object):
+            d = id(object)
+            memo = self.memo
+            unicode = object.isunicode()
+
+            if self.bin:
+                if unicode:
+                    object = object.encode("utf-8")
+                l = len(object)
+                s = mdumps(l)[1:]
+                if l < 256 and not unicode:
+                    self.write(SHORT_BINSTRING + s[0] + object)
+                else:
+                    if unicode:
+                        self.write(BINUNICODE + s + object)
+                    else:
+                        self.write(BINSTRING + s + object)
+            else:
+                if unicode:
+                    object = object.replace("\\", "\\u005c")
+                    object = object.replace("\n", "\\u000a")
+                    object = object.encode('raw-unicode-escape')
+                    self.write(UNICODE + object + '\n')
+                else:
+                    self.write(STRING + `object` + '\n')
+
+            memo_len = len(memo)
+            self.write(self.put(memo_len))
+            memo[d] = (memo_len, object)
+        dispatch[StringType] = save_string
+
     def save_tuple(self, object):
 
         write = self.write
@@ -310,11 +373,11 @@ class Pickler:
         for element in object:
             save(element)
 
-        if (len(object) and memo.has_key(d)):
-            if (self.bin):
+        if len(object) and memo.has_key(d):
+            if self.bin:
                 write(POP_MARK + self.get(memo[d][0]))
                 return
-           
+
             write(POP * (len(object) + 1) + self.get(memo[d][0]))
             return
 
@@ -333,7 +396,7 @@ class Pickler:
         save  = self.save
         memo  = self.memo
 
-        if (self.bin):
+        if self.bin:
             write(EMPTY_LIST)
         else:
             write(MARK + LIST)
@@ -344,16 +407,16 @@ class Pickler:
 
         using_appends = (self.bin and (len(object) > 1))
 
-        if (using_appends):
+        if using_appends:
             write(MARK)
 
         for element in object:
             save(element)
-  
-            if (not using_appends):
+
+            if not using_appends:
                 write(APPEND)
 
-        if (using_appends):
+        if using_appends:
             write(APPENDS)
     dispatch[ListType] = save_list
 
@@ -364,7 +427,7 @@ class Pickler:
         save  = self.save
         memo  = self.memo
 
-        if (self.bin):
+        if self.bin:
             write(EMPTY_DICT)
         else:
             write(MARK + DICT)
@@ -375,7 +438,7 @@ class Pickler:
 
         using_setitems = (self.bin and (len(object) > 1))
 
-        if (using_setitems):
+        if using_setitems:
             write(MARK)
 
         items = object.items()
@@ -383,10 +446,10 @@ class Pickler:
             save(key)
             save(value)
 
-            if (not using_setitems):
+            if not using_setitems:
                 write(SETITEM)
 
-        if (using_setitems):
+        if using_setitems:
             write(SETITEMS)
 
     dispatch[DictionaryType] = save_dict
@@ -410,14 +473,14 @@ class Pickler:
 
         write(MARK)
 
-        if (self.bin):
+        if self.bin:
             save(cls)
 
         for arg in args:
             save(arg)
 
         memo_len = len(memo)
-        if (self.bin):
+        if self.bin:
             write(OBJ + self.put(memo_len))
         else:
             write(INST + cls.__module__ + '\n' + cls.__name__ + '\n' +
@@ -440,7 +503,7 @@ class Pickler:
         write = self.write
         memo = self.memo
 
-        if (name is None):
+        if name is None:
             name = object.__name__
 
         try:
@@ -448,6 +511,20 @@ class Pickler:
         except AttributeError:
             module = whichmodule(object, name)
 
+        try:
+            __import__(module)
+            mod = sys.modules[module]
+            klass = getattr(mod, name)
+        except (ImportError, KeyError, AttributeError):
+            raise PicklingError(
+                "Can't pickle %r: it's not found as %s.%s" %
+                (object, module, name))
+        else:
+            if klass is not object:
+                raise PicklingError(
+                    "Can't pickle %r: it's not the same object as %s.%s" %
+                    (object, module, name))
+
         memo_len = len(memo)
         write(GLOBAL + module + '\n' + name + '\n' +
             self.put(memo_len))
@@ -455,6 +532,7 @@ class Pickler:
     dispatch[ClassType] = save_global
     dispatch[FunctionType] = save_global
     dispatch[BuiltinFunctionType] = save_global
+    dispatch[TypeType] = save_global
 
 
 def _keep_alive(x, memo):
@@ -507,7 +585,7 @@ class Unpickler:
         self.memo = {}
 
     def load(self):
-        self.mark = ['spam'] # Any new unique object
+        self.mark = object() # any new unique object
         self.stack = []
         self.append = self.stack.append
         read = self.read
@@ -516,8 +594,8 @@ class Unpickler:
             while 1:
                 key = read(1)
                 dispatch[key](self)
-        except STOP, value:
-            return value
+        except _Stop, stopinst:
+            return stopinst.value
 
     def marker(self):
         stack = self.stack
@@ -539,7 +617,7 @@ class Unpickler:
 
     def load_binpersid(self):
         stack = self.stack
-         
+
         pid = stack[-1]
         del stack[-1]
 
@@ -551,7 +629,11 @@ class Unpickler:
     dispatch[NONE] = load_none
 
     def load_int(self):
-        self.append(string.atoi(self.readline()[:-1]))
+        data = self.readline()
+        try:
+            self.append(int(data))
+        except ValueError:
+            self.append(long(data))
     dispatch[INT] = load_int
 
     def load_binint(self):
@@ -565,13 +647,13 @@ class Unpickler:
     def load_binint2(self):
         self.append(mloads('i' + self.read(2) + '\000\000'))
     dispatch[BININT2] = load_binint2
- 
+
     def load_long(self):
-        self.append(string.atol(self.readline()[:-1], 0))
+        self.append(long(self.readline()[:-1], 0))
     dispatch[LONG] = load_long
 
     def load_float(self):
-        self.append(string.atof(self.readline()[:-1]))
+        self.append(float(self.readline()[:-1]))
     dispatch[FLOAT] = load_float
 
     def load_binfloat(self, unpack=struct.unpack):
@@ -695,6 +777,9 @@ class Unpickler:
                 pass
         if not instantiated:
             try:
+                if not hasattr(klass, '__safe_for_unpickling__'):
+                    raise UnpicklingError('%s is not safe for unpickling' %
+                                          klass)
                 value = apply(klass, args)
             except TypeError, err:
                 raise TypeError, "in constructor for %s: %s" % (
@@ -707,7 +792,7 @@ class Unpickler:
         k = self.marker()
         klass = stack[k + 1]
         del stack[k + 1]
-        args = tuple(stack[k + 1:]) 
+        args = tuple(stack[k + 1:])
         del stack[k:]
         instantiated = 0
         if (not args and type(klass) is ClassType and
@@ -723,7 +808,7 @@ class Unpickler:
         if not instantiated:
             value = apply(klass, args)
         self.append(value)
-    dispatch[OBJ] = load_obj                
+    dispatch[OBJ] = load_obj
 
     def load_global(self):
         module = self.readline()[:-1]
@@ -733,14 +818,9 @@ class Unpickler:
     dispatch[GLOBAL] = load_global
 
     def find_class(self, module, name):
-        try:
-            __import__(module)
-            mod = sys.modules[module]
-            klass = getattr(mod, name)
-        except (ImportError, KeyError, AttributeError):
-            raise SystemError, \
-                  "Failed to import class %s from module %s" % \
-                  (name, module)
+        __import__(module)
+        mod = sys.modules[module]
+        klass = getattr(mod, name)
         return klass
 
     def load_reduce(self):
@@ -757,9 +837,9 @@ class Unpickler:
                 except AttributeError:
                     safe = None
 
-                if (not safe):
-                   raise UnpicklingError, "%s is not safe for " \
-                                          "unpickling" % callable
+                if not safe:
+                    raise UnpicklingError, "%s is not safe for " \
+                                           "unpickling" % callable
 
         if arg_tup is None:
             value = callable.__basicnew__()
@@ -826,7 +906,7 @@ class Unpickler:
 
         del stack[mark:]
     dispatch[APPENDS] = load_appends
-           
+
     def load_setitem(self):
         stack = self.stack
         value = stack[-1]
@@ -874,7 +954,7 @@ class Unpickler:
     def load_stop(self):
         value = self.stack[-1]
         del self.stack[-1]
-        raise STOP, value
+        raise _Stop(value)
     dispatch[STOP] = load_stop
 
 # Helper class for load_inst/load_obj
@@ -884,7 +964,10 @@ class _EmptyClass:
 
 # Shorthands
 
-from StringIO import StringIO
+try:
+    from cStringIO import StringIO
+except ImportError:
+    from StringIO import StringIO
 
 def dump(object, file, bin = 0):
     Pickler(file, bin).dump(object)
@@ -900,38 +983,3 @@ def load(file):
 def loads(str):
     file = StringIO(str)
     return Unpickler(file).load()
-
-
-# The rest is used for testing only
-
-class C:
-    def __cmp__(self, other):
-        return cmp(self.__dict__, other.__dict__)
-
-def test():
-    fn = 'out'
-    c = C()
-    c.foo = 1
-    c.bar = 2
-    x = [0, 1, 2, 3]
-    y = ('abc', 'abc', c, c)
-    x.append(y)
-    x.append(y)
-    x.append(5)
-    f = open(fn, 'w')
-    F = Pickler(f)
-    F.dump(x)
-    f.close()
-    f = open(fn, 'r')
-    U = Unpickler(f)
-    x2 = U.load()
-    print x
-    print x2
-    print x == x2
-    print map(id, x)
-    print map(id, x2)
-    print F.memo
-    print U.memo
-
-if __name__ == '__main__':
-    test()
