@@ -2,13 +2,14 @@
 /* Time module */
 
 #include "Python.h"
+#include "structseq.h"
 
 #include <ctype.h>
 
 #ifdef macintosh
 #include <time.h>
 #include <OSUtils.h>
-#ifdef USE_GUSI2
+#ifdef USE_GUSI211
 /* GUSI, the I/O library which has the time() function and such uses the
 ** Mac epoch of 1904. MSL, the C library which has localtime() and so uses
 ** the ANSI epoch of 1900.
@@ -23,10 +24,6 @@
 #include <io.h>
 #endif
 
-#ifdef HAVE_UNISTD_H
-#include <unistd.h>
-#endif
-
 #ifdef HAVE_FTIME
 #include <sys/timeb.h>
 #if !defined(MS_WINDOWS) && !defined(PYOS_OS2)
@@ -39,17 +36,19 @@ extern int ftime(struct timeb *);
 #else
 #ifdef MS_WINDOWS
 #include <windows.h>
-#ifdef MS_WIN16
+#if defined(MS_WIN16) || defined(__BORLANDC__)
 /* These overrides not needed for Win32 */
 #define timezone _timezone
 #define tzname _tzname
 #define daylight _daylight
+#endif /* MS_WIN16 || __BORLANDC__ */
+#ifdef MS_WIN16
 #define altzone _altzone
 #endif /* MS_WIN16 */
 #endif /* MS_WINDOWS */
 #endif /* !__WATCOMC__ || __QNX__ */
 
-#if defined(MS_WIN32) && !defined(MS_WIN64)
+#if defined(MS_WIN32) && !defined(MS_WIN64) && !defined(__BORLANDC__)
 /* Win32 has better clock replacement
    XXX Win64 does not yet, but might when the platform matures. */
 #include <largeint.h>
@@ -84,22 +83,22 @@ static PyObject *moddict;
 */
 static long timezone;
 
-static void 
+static void
 initmactimezone(void)
 {
 	MachineLocation	loc;
 	long		delta;
 
 	ReadLocation(&loc);
-	
+
 	if (loc.latitude == 0 && loc.longitude == 0 && loc.u.gmtDelta == 0)
 		return;
-	
+
 	delta = loc.u.gmtDelta & 0x00FFFFFF;
-	
+
 	if (delta & 0x00800000)
 		delta |= 0xFF000000;
-	
+
 	timezone = -delta;
 }
 #endif /* macintosh */
@@ -109,7 +108,7 @@ static PyObject *
 time_time(PyObject *self, PyObject *args)
 {
 	double secs;
-	if (!PyArg_NoArgs(args))
+	if (!PyArg_ParseTuple(args, ":time"))
 		return NULL;
 	secs = floattime();
 	if (secs == 0.0) {
@@ -138,13 +137,13 @@ Fractions of a second may be present if the system clock provides them.";
 static PyObject *
 time_clock(PyObject *self, PyObject *args)
 {
-	if (!PyArg_NoArgs(args))
+	if (!PyArg_ParseTuple(args, ":clock"))
 		return NULL;
 	return PyFloat_FromDouble(((double)clock()) / CLOCKS_PER_SEC);
 }
 #endif /* HAVE_CLOCK */
 
-#if defined(MS_WIN32) && !defined(MS_WIN64)
+#if defined(MS_WIN32) && !defined(MS_WIN64) && !defined(__BORLANDC__)
 /* Due to Mark Hammond */
 static PyObject *
 time_clock(PyObject *self, PyObject *args)
@@ -153,15 +152,15 @@ time_clock(PyObject *self, PyObject *args)
 	static LARGE_INTEGER divisor = {0,0};
 	LARGE_INTEGER now, diff, rem;
 
-	if (!PyArg_NoArgs(args))
+	if (!PyArg_ParseTuple(args, ":clock"))
 		return NULL;
 
 	if (LargeIntegerEqualToZero(divisor)) {
 		QueryPerformanceCounter(&ctrStart);
-		if (!QueryPerformanceFrequency(&divisor) || 
+		if (!QueryPerformanceFrequency(&divisor) ||
 		    LargeIntegerEqualToZero(divisor)) {
-				/* Unlikely to happen - 
-				   this works on all intel machines at least! 
+				/* Unlikely to happen -
+				   this works on all intel machines at least!
 				   Revert to clock() */
 			return PyFloat_FromDouble(clock());
 		}
@@ -170,10 +169,10 @@ time_clock(PyObject *self, PyObject *args)
 	diff = LargeIntegerSubtract(now, ctrStart);
 	diff = LargeIntegerDivide(diff, divisor, &rem);
 	/* XXX - we assume both divide results fit in 32 bits.  This is
-	   true on Intels.  First person who can afford a machine that 
+	   true on Intels.  First person who can afford a machine that
 	   doesnt deserves to fix it :-)
 	*/
-	return PyFloat_FromDouble((double)diff.LowPart + 
+	return PyFloat_FromDouble((double)diff.LowPart +
 		              ((double)rem.LowPart / (double)divisor.LowPart));
 }
 
@@ -192,7 +191,7 @@ static PyObject *
 time_sleep(PyObject *self, PyObject *args)
 {
 	double secs;
-	if (!PyArg_Parse(args, "d", &secs))
+	if (!PyArg_ParseTuple(args, "d:sleep", &secs))
 		return NULL;
 	if (floatsleep(secs) != 0)
 		return NULL;
@@ -206,19 +205,53 @@ static char sleep_doc[] =
 Delay execution for a given number of seconds.  The argument may be\n\
 a floating point number for subsecond precision.";
 
+static PyStructSequence_Field struct_time_type_fields[] = {
+	{"tm_year", NULL},
+	{"tm_mon", NULL},
+	{"tm_mday", NULL},
+	{"tm_hour", NULL},
+	{"tm_min", NULL},
+	{"tm_sec", NULL},
+	{"tm_wday", NULL},
+	{"tm_yday", NULL},
+	{"tm_isdst", NULL},
+	{0}
+};
+
+static PyStructSequence_Desc struct_time_type_desc = {
+	"time.struct_time",
+	NULL,
+	struct_time_type_fields,
+	9,
+};
+	
+static PyTypeObject StructTimeType;
+
 static PyObject *
 tmtotuple(struct tm *p)
 {
-	return Py_BuildValue("(iiiiiiiii)",
-			     p->tm_year + 1900,
-			     p->tm_mon + 1,	   /* Want January == 1 */
-			     p->tm_mday,
-			     p->tm_hour,
-			     p->tm_min,
-			     p->tm_sec,
-			     (p->tm_wday + 6) % 7, /* Want Monday == 0 */
-			     p->tm_yday + 1,	   /* Want January, 1 == 1 */
-			     p->tm_isdst);
+	PyObject *v = PyStructSequence_New(&StructTimeType);
+	if (v == NULL)
+		return NULL;
+	
+#define SET(i,val) PyStructSequence_SET_ITEM(v, i, PyInt_FromLong((long) val))
+
+	SET(0, p->tm_year + 1900);
+	SET(1, p->tm_mon + 1);	   /* Want January == 1 */
+	SET(2, p->tm_mday);
+	SET(3, p->tm_hour);
+	SET(4, p->tm_min);
+	SET(5, p->tm_sec);
+	SET(6, (p->tm_wday + 6) % 7); /* Want Monday == 0 */
+	SET(7, p->tm_yday + 1);	   /* Want January, 1 == 1 */
+	SET(8, p->tm_isdst);
+#undef SET
+	if (PyErr_Occurred()) {
+		Py_XDECREF(v);
+		return NULL;
+	}
+
+	return v;
 }
 
 static PyObject *
@@ -244,28 +277,36 @@ static PyObject *
 time_gmtime(PyObject *self, PyObject *args)
 {
 	double when;
-	if (!PyArg_Parse(args, "d", &when))
+	if (PyTuple_Size(args) == 0)
+		when = floattime();
+	if (!PyArg_ParseTuple(args, "|d:gmtime", &when))
 		return NULL;
 	return time_convert((time_t)when, gmtime);
 }
 
 static char gmtime_doc[] =
-"gmtime(seconds) -> tuple\n\
+"gmtime([seconds]) -> (tm_year, tm_mon, tm_day, tm_hour, tm_min,\n\
+                       tm_sec, tm_wday, tm_yday, tm_isdst)\n\
 \n\
-Convert seconds since the Epoch to a time tuple expressing UTC (a.k.a. GMT).";
+Convert seconds since the Epoch to a time tuple expressing UTC (a.k.a.\n\
+GMT).  When 'seconds' is not passed in, convert the current time instead.";
 
 static PyObject *
 time_localtime(PyObject *self, PyObject *args)
 {
 	double when;
-	if (!PyArg_Parse(args, "d", &when))
+	if (PyTuple_Size(args) == 0)
+		when = floattime();
+	if (!PyArg_ParseTuple(args, "|d:localtime", &when))
 		return NULL;
 	return time_convert((time_t)when, localtime);
 }
 
 static char localtime_doc[] =
-"localtime(seconds) -> tuple\n\
-Convert seconds since the Epoch to a time tuple expressing local time.";
+"localtime([seconds]) -> (tm_year,tm_mon,tm_day,tm_hour,tm_min,tm_sec,tm_wday,tm_yday,tm_isdst)\n\
+\n\
+Convert seconds since the Epoch to a time tuple expressing local time.\n\
+When 'seconds' is not passed in, convert the current time instead.";
 
 static int
 gettmarg(PyObject *args, struct tm *p)
@@ -299,7 +340,7 @@ gettmarg(PyObject *args, struct tm *p)
 			y += 2000;
 		else {
 			PyErr_SetString(PyExc_ValueError,
-					"year out of range (00-99, 1900-*)");
+					"year out of range");
 			return 0;
 		}
 	}
@@ -314,7 +355,7 @@ gettmarg(PyObject *args, struct tm *p)
 static PyObject *
 time_strftime(PyObject *self, PyObject *args)
 {
-	PyObject *tup;
+	PyObject *tup = NULL;
 	struct tm buf;
 	const char *fmt;
 	size_t fmtlen, buflen;
@@ -323,9 +364,15 @@ time_strftime(PyObject *self, PyObject *args)
 
 	memset((void *) &buf, '\0', sizeof(buf));
 
-	if (!PyArg_ParseTuple(args, "sO:strftime", &fmt, &tup) 
-	    || !gettmarg(tup, &buf))
+	if (!PyArg_ParseTuple(args, "s|O:strftime", &fmt, &tup))
 		return NULL;
+
+	if (tup == NULL) {
+		time_t tt = time(NULL);
+		buf = *localtime(&tt);
+	} else if (!gettmarg(tup, &buf))
+		return NULL;
+
 	fmtlen = strlen(fmt);
 
 	/* I hate these functions that presume you know how big the output
@@ -353,10 +400,11 @@ time_strftime(PyObject *self, PyObject *args)
 }
 
 static char strftime_doc[] =
-"strftime(format, tuple) -> string\n\
+"strftime(format[, tuple]) -> string\n\
 \n\
 Convert a time tuple to a string according to a format specification.\n\
-See the library reference manual for formatting codes.";
+See the library reference manual for formatting codes. When the time tuple\n\
+is not present, current time as returned by localtime() is used.";
 #endif /* HAVE_STRFTIME */
 
 #ifdef HAVE_STRPTIME
@@ -394,6 +442,7 @@ time_strptime(PyObject *self, PyObject *args)
 
 static char strptime_doc[] =
 "strptime(string, format) -> tuple\n\
+\n\
 Parse a string to a time tuple according to a format specification.\n\
 See the library reference manual for formatting codes (same as strftime()).";
 #endif /* HAVE_STRPTIME */
@@ -401,12 +450,15 @@ See the library reference manual for formatting codes (same as strftime()).";
 static PyObject *
 time_asctime(PyObject *self, PyObject *args)
 {
-	PyObject *tup;
+	PyObject *tup = NULL;
 	struct tm buf;
 	char *p;
-	if (!PyArg_ParseTuple(args, "O:asctime", &tup))
+	if (!PyArg_ParseTuple(args, "|O:asctime", &tup))
 		return NULL;
-	if (!gettmarg(tup, &buf))
+	if (tup == NULL) {
+		time_t tt = time(NULL);
+		buf = *localtime(&tt);
+	} else if (!gettmarg(tup, &buf))
 		return NULL;
 	p = asctime(&buf);
 	if (p[24] == '\n')
@@ -415,9 +467,11 @@ time_asctime(PyObject *self, PyObject *args)
 }
 
 static char asctime_doc[] =
-"asctime(tuple) -> string\n\
+"asctime([tuple]) -> string\n\
 \n\
-Convert a time tuple to a string, e.g. 'Sat Jun 06 16:26:11 1998'.";
+Convert a time tuple to a string, e.g. 'Sat Jun 06 16:26:11 1998'.\n\
+When the time tuple is not present, current time as returned by localtime()\n\
+is used.";
 
 static PyObject *
 time_ctime(PyObject *self, PyObject *args)
@@ -425,9 +479,14 @@ time_ctime(PyObject *self, PyObject *args)
 	double dt;
 	time_t tt;
 	char *p;
-	if (!PyArg_Parse(args, "d", &dt))
-		return NULL;
-	tt = (time_t)dt;
+
+	if (PyTuple_Size(args) == 0)
+		tt = time(NULL);
+	else {
+		if (!PyArg_ParseTuple(args, "|d:ctime", &dt))
+			return NULL;
+		tt = (time_t)dt;
+	}
 #if defined(macintosh) && defined(USE_GUSI204)
 	tt = tt + GUSI_TO_MSL_EPOCH;
 #endif
@@ -445,7 +504,8 @@ static char ctime_doc[] =
 "ctime(seconds) -> string\n\
 \n\
 Convert a time in seconds since the Epoch to a string in local time.\n\
-This is equivalent to asctime(localtime(seconds)).";
+This is equivalent to asctime(localtime(seconds)). When the time tuple is\n\
+not present, current time as returned by localtime() is used.";
 
 #ifdef HAVE_MKTIME
 static PyObject *
@@ -463,10 +523,10 @@ time_mktime(PyObject *self, PyObject *args)
 	tt = mktime(&buf);
 	if (tt == (time_t)(-1)) {
 		PyErr_SetString(PyExc_OverflowError,
-                                "mktime argument out of range");
+				"mktime argument out of range");
 		return NULL;
 	}
-#if defined(macintosh) && defined(USE_GUSI2)
+#if defined(macintosh) && defined(USE_GUSI211)
 	tt = tt - GUSI_TO_MSL_EPOCH;
 #endif
 	return PyFloat_FromDouble((double)tt);
@@ -479,15 +539,15 @@ Convert a time tuple in local time to seconds since the Epoch.";
 #endif /* HAVE_MKTIME */
 
 static PyMethodDef time_methods[] = {
-	{"time",	time_time, METH_OLDARGS, time_doc},
+	{"time",	time_time, METH_VARARGS, time_doc},
 #ifdef HAVE_CLOCK
-	{"clock",	time_clock, METH_OLDARGS, clock_doc},
+	{"clock",	time_clock, METH_VARARGS, clock_doc},
 #endif
-	{"sleep",	time_sleep, METH_OLDARGS, sleep_doc},
-	{"gmtime",	time_gmtime, METH_OLDARGS, gmtime_doc},
-	{"localtime",	time_localtime, METH_OLDARGS, localtime_doc},
+	{"sleep",	time_sleep, METH_VARARGS, sleep_doc},
+	{"gmtime",	time_gmtime, METH_VARARGS, gmtime_doc},
+	{"localtime",	time_localtime, METH_VARARGS, localtime_doc},
 	{"asctime",	time_asctime, METH_VARARGS, asctime_doc},
-	{"ctime",	time_ctime, METH_OLDARGS, ctime_doc},
+	{"ctime",	time_ctime, METH_VARARGS, ctime_doc},
 #ifdef HAVE_MKTIME
 	{"mktime",	time_mktime, METH_VARARGS, mktime_doc},
 #endif
@@ -556,7 +616,7 @@ mktime() -- convert local time tuple to seconds since Epoch\n\
 strftime() -- convert time tuple to string according to format specification\n\
 strptime() -- parse string to time tuple according to format specification\n\
 ";
-  
+
 
 DL_EXPORT(void)
 inittime(void)
@@ -566,12 +626,12 @@ inittime(void)
 	m = Py_InitModule3("time", time_methods, module_doc);
 	d = PyModule_GetDict(m);
 	/* Accept 2-digit dates unless PYTHONY2K is set and non-empty */
-	p = getenv("PYTHONY2K");
+	p = Py_GETENV("PYTHONY2K");
 	ins(d, "accept2dyear", PyInt_FromLong((long) (!p || !*p)));
 	/* Squirrel away the module's dictionary for the y2k check */
 	Py_INCREF(d);
 	moddict = d;
-#if defined(HAVE_TZNAME) && !defined(__GLIBC__)
+#if defined(HAVE_TZNAME) && !defined(__GLIBC__) && !defined(__CYGWIN__)
 	tzset();
 #ifdef PYOS_OS2
 	ins(d, "timezone", PyInt_FromLong((long)_timezone));
@@ -589,7 +649,7 @@ inittime(void)
 #endif
 	ins(d, "daylight", PyInt_FromLong((long)daylight));
 	ins(d, "tzname", Py_BuildValue("(zz)", tzname[0], tzname[1]));
-#else /* !HAVE_TZNAME || __GLIBC__ */
+#else /* !HAVE_TZNAME || __GLIBC__ || __CYGWIN__*/
 #ifdef HAVE_TM_ZONE
 	{
 #define YEAR ((time_t)((365 * 24 + 6) * 3600))
@@ -607,7 +667,7 @@ inittime(void)
 		julyzone = -p->tm_gmtoff;
 		strncpy(julyname, p->tm_zone ? p->tm_zone : "   ", 9);
 		julyname[9] = '\0';
-		
+
 		if( janzone < julyzone ) {
 			/* DST is reversed in the southern hemisphere */
 			ins(d, "timezone", PyInt_FromLong(julyzone));
@@ -638,7 +698,17 @@ inittime(void)
 	ins(d, "tzname", Py_BuildValue("(zz)", "", ""));
 #endif /* macintosh */
 #endif /* HAVE_TM_ZONE */
-#endif /* !HAVE_TZNAME || __GLIBC__ */
+#ifdef __CYGWIN__
+	tzset();
+	ins(d, "timezone", PyInt_FromLong(_timezone));
+	ins(d, "altzone", PyInt_FromLong(_timezone));
+	ins(d, "daylight", PyInt_FromLong(_daylight));
+	ins(d, "tzname", Py_BuildValue("(zz)", _tzname[0], _tzname[1]));
+#endif /* __CYGWIN__ */
+#endif /* !HAVE_TZNAME || __GLIBC__ || __CYGWIN__*/
+
+        PyStructSequence_InitType(&StructTimeType, &struct_time_type_desc);
+	PyDict_SetItemString(d, "struct_time", (PyObject*) &StructTimeType);
 }
 
 
@@ -785,7 +855,7 @@ floatsleep(double secs)
 		if( secs <= 0.0 ) {
 			return;
 		}
-		
+
 		Py_BEGIN_ALLOW_THREADS
 		/* BeOS snooze() is in microseconds... */
 		if( snooze( (bigtime_t)( secs * 1000.0 * 1000.0 ) ) == B_INTERRUPTED ) {
@@ -796,10 +866,20 @@ floatsleep(double secs)
 		Py_END_ALLOW_THREADS
 	}
 #else /* !__BEOS__ */
+#ifdef RISCOS
+	if (secs <= 0.0)
+		return 0;
+	Py_BEGIN_ALLOW_THREADS
+	/* This sleep *CAN BE* interrupted. */
+	if ( sleep(secs) )
+		return -1;
+	Py_END_ALLOW_THREADS
+#else /* !RISCOS */
 	/* XXX Can't interrupt this sleep */
 	Py_BEGIN_ALLOW_THREADS
 	sleep((int)secs);
 	Py_END_ALLOW_THREADS
+#endif /* !RISCOS */
 #endif /* !__BEOS__ */
 #endif /* !PYOS_OS2 */
 #endif /* !MS_WIN32 */
@@ -807,5 +887,6 @@ floatsleep(double secs)
 #endif /* !__WATCOMC__ || __QNX__ */
 #endif /* !macintosh */
 #endif /* !HAVE_SELECT */
+
 	return 0;
 }
