@@ -26,7 +26,7 @@
  * compile-time literal concatenation.
  */
 static char
-module__doc__[] = 
+module__doc__[] =
 "Python's standard exception class hierarchy.\n\
 \n\
 Before Python 1.5, the standard exceptions were all simple string objects.\n\
@@ -52,52 +52,62 @@ recommended that user defined class based exceptions be derived from the\n\
 Exception\n\
  |\n\
  +-- SystemExit\n\
+ +-- StopIteration\n\
  +-- StandardError\n\
+ |    |\n\
+ |    +-- KeyboardInterrupt\n\
+ |    +-- ImportError\n\
+ |    +-- EnvironmentError\n\
+ |    |    |\n\
+ |    |    +-- IOError\n\
+ |    |    +-- OSError\n\
+ |    |         |\n\
+ |    |         +-- WindowsError\n\
+ |    |\n\
+ |    +-- EOFError\n\
+ |    +-- RuntimeError\n\
+ |    |    |\n\
+ |    |    +-- NotImplementedError\n\
+ |    |\n\
+ |    +-- NameError\n\
+ |    |    |\n\
+ |    |    +-- UnboundLocalError\n\
+ |    |\n\
+ |    +-- AttributeError\n\
+ |    +-- SyntaxError\n\
+ |    |    |\n\
+ |    |    +-- IndentationError\n\
+ |    |         |\n\
+ |    |         +-- TabError\n\
+ |    |\n\
+ |    +-- TypeError\n\
+ |    +-- AssertionError\n\
+ |    +-- LookupError\n\
+ |    |    |\n\
+ |    |    +-- IndexError\n\
+ |    |    +-- KeyError\n\
+ |    |\n\
+ |    +-- ArithmeticError\n\
+ |    |    |\n\
+ |    |    +-- OverflowError\n\
+ |    |    +-- ZeroDivisionError\n\
+ |    |    +-- FloatingPointError\n\
+ |    |\n\
+ |    +-- ValueError\n\
+ |    |    |\n\
+ |    |    +-- UnicodeError\n\
+ |    |\n\
+ |    +-- ReferenceError\n\
+ |    +-- SystemError\n\
+ |    +-- MemoryError\n\
+ |\n\
+ +---Warning\n\
       |\n\
-      +-- KeyboardInterrupt\n\
-      +-- ImportError\n\
-      +-- EnvironmentError\n\
-      |    |\n\
-      |    +-- IOError\n\
-      |    +-- OSError\n\
-      |         |\n\
-      |         +-- WindowsError\n\
-      |\n\
-      +-- EOFError\n\
-      +-- RuntimeError\n\
-      |    |\n\
-      |    +-- NotImplementedError\n\
-      |\n\
-      +-- NameError\n\
-      |    |\n\
-      |    +-- UnboundLocalError\n\
-      |\n\
-      +-- AttributeError\n\
-      +-- SyntaxError\n\
-      |    |\n\
-      |    +-- IndentationError\n\
-      |         |\n\
-      |         +-- TabError\n\
-      |\n\
-      +-- TypeError\n\
-      +-- AssertionError\n\
-      +-- LookupError\n\
-      |    |\n\
-      |    +-- IndexError\n\
-      |    +-- KeyError\n\
-      |\n\
-      +-- ArithmeticError\n\
-      |    |\n\
-      |    +-- OverflowError\n\
-      |    +-- ZeroDivisionError\n\
-      |    +-- FloatingPointError\n\
-      |\n\
-      +-- ValueError\n\
-      |    |\n\
-      |    +-- UnicodeError\n\
-      |\n\
-      +-- SystemError\n\
-      +-- MemoryError";
+      +-- UserWarning\n\
+      +-- DeprecationWarning\n\
+      +-- SyntaxWarning\n\
+      +-- OverflowWarning\n\
+      +-- RuntimeWarning";
 
 
 /* Helper function for populating a dictionary with method wrappers. */
@@ -121,7 +131,7 @@ populate_methods(PyObject *klass, PyObject *dict, PyMethodDef *methods)
 	    Py_DECREF(func);
 	    return -1;
 	}
-	
+
 	/* add method to dictionary */
 	status = PyDict_SetItemString(dict, methods->ml_name, meth);
 	Py_DECREF(meth);
@@ -136,7 +146,7 @@ populate_methods(PyObject *klass, PyObject *dict, PyMethodDef *methods)
     return 0;
 }
 
-	
+
 
 /* This function is used to create all subsequent exception classes. */
 static int
@@ -189,7 +199,7 @@ get_self(PyObject *args)
 	/* Watch out for being called to early in the bootstrapping process */
 	if (PyExc_TypeError) {
 	    PyErr_SetString(PyExc_TypeError,
-	     "unbound method must be called with class instance 1st argument");
+	     "unbound method must be called with instance as first argument");
 	}
         return NULL;
     }
@@ -334,7 +344,7 @@ make_Exception(char *modulename)
 
     if (!(name = PyString_FromString("Exception")))
 	goto finally;
-    
+
     if (!(PyExc_Exception = PyClass_New(NULL, dict, name)))
 	goto finally;
 
@@ -362,6 +372,9 @@ StandardError__doc__[] = "Base class for all standard Python exceptions.";
 static char
 TypeError__doc__[] = "Inappropriate argument type.";
 
+static char
+StopIteration__doc__[] = "Signal the end from iterator.next().";
+
 
 
 static char
@@ -380,7 +393,7 @@ SystemExit__init__(PyObject *self, PyObject *args)
     /* Set args attribute. */
     if (!(args = PySequence_GetSlice(args, 1, PySequence_Size(args))))
         return NULL;
-    
+
     status = PyObject_SetAttrString(self, "args", args);
     if (status < 0) {
 	Py_DECREF(args);
@@ -413,7 +426,7 @@ SystemExit__init__(PyObject *self, PyObject *args)
 }
 
 
-PyMethodDef SystemExit_methods[] = {
+static PyMethodDef SystemExit_methods[] = {
     { "__init__", SystemExit__init__, METH_VARARGS},
     {NULL, NULL}
 };
@@ -462,9 +475,9 @@ EnvironmentError__init__(PyObject *self, PyObject *args)
 	 * of the os module functions, PyErr_SetFromErrnoWithFilename() is
 	 * called, giving a third argument which is the filename.  But, so
 	 * that old code using in-place unpacking doesn't break, e.g.:
-	 * 
+	 *
 	 * except IOError, (errno, strerror):
-	 * 
+	 *
 	 * we hack args so that it only contains two items.  This also
 	 * means we need our own __str__() which prints out the filename
 	 * when it was supplied.
@@ -474,7 +487,7 @@ EnvironmentError__init__(PyObject *self, PyObject *args)
 	item2 = PySequence_GetItem(args, 2);
 	if (!item0 || !item1 || !item2)
 	    goto finally;
-	
+
 	if (PyObject_SetAttrString(self, "errno", item0) ||
 	    PyObject_SetAttrString(self, "strerror", item1) ||
 	    PyObject_SetAttrString(self, "filename", item2))
@@ -495,7 +508,7 @@ EnvironmentError__init__(PyObject *self, PyObject *args)
 	item1 = PySequence_GetItem(args, 1);
 	if (!item0 || !item1)
 	    goto finally;
-	
+
 	if (PyObject_SetAttrString(self, "errno", item0) ||
 	    PyObject_SetAttrString(self, "strerror", item1))
 	{
@@ -528,7 +541,7 @@ EnvironmentError__str__(PyObject *self, PyObject *args)
 
     if (!PyArg_ParseTuple(args, "O:__str__", &self))
 	return NULL;
-    
+
     filename = PyObject_GetAttrString(self, "filename");
     serrno = PyObject_GetAttrString(self, "errno");
     strerror = PyObject_GetAttrString(self, "strerror");
@@ -571,7 +584,7 @@ EnvironmentError__str__(PyObject *self, PyObject *args)
 
 	PyTuple_SET_ITEM(tuple, 0, serrno);
 	PyTuple_SET_ITEM(tuple, 1, strerror);
-	
+
 	rtnval = PyString_Format(fmt, tuple);
 
 	Py_DECREF(fmt);
@@ -657,7 +670,8 @@ SyntaxError__classinit__(PyObject *klass)
 	PyObject_SetAttrString(klass, "filename", Py_None) ||
 	PyObject_SetAttrString(klass, "lineno", Py_None) ||
 	PyObject_SetAttrString(klass, "offset", Py_None) ||
-	PyObject_SetAttrString(klass, "text", Py_None))
+	PyObject_SetAttrString(klass, "text", Py_None) ||
+	PyObject_SetAttrString(klass, "print_file_and_line", Py_None))
     {
 	retval = -1;
     }
@@ -695,29 +709,35 @@ SyntaxError__init__(PyObject *self, PyObject *args)
     }
     if (lenargs == 2) {
 	PyObject *info = PySequence_GetItem(args, 1);
-	PyObject *filename, *lineno, *offset, *text;
+	PyObject *filename = NULL, *lineno = NULL;
+	PyObject *offset = NULL, *text = NULL;
 	int status = 1;
 
 	if (!info)
 	    goto finally;
 
 	filename = PySequence_GetItem(info, 0);
-	lineno = PySequence_GetItem(info, 1);
-	offset = PySequence_GetItem(info, 2);
-	text = PySequence_GetItem(info, 3);
-
-	Py_DECREF(info);
-
-	if (filename && lineno && offset && text) {
-	    status = PyObject_SetAttrString(self, "filename", filename) ||
-		PyObject_SetAttrString(self, "lineno", lineno) ||
-		PyObject_SetAttrString(self, "offset", offset) ||
-		PyObject_SetAttrString(self, "text", text);
+	if (filename != NULL) {
+	    lineno = PySequence_GetItem(info, 1);
+	    if (lineno != NULL) {
+		offset = PySequence_GetItem(info, 2);
+		if (offset != NULL) {
+		    text = PySequence_GetItem(info, 3);
+		    if (text != NULL) {
+			status =
+			    PyObject_SetAttrString(self, "filename", filename)
+			    || PyObject_SetAttrString(self, "lineno", lineno)
+			    || PyObject_SetAttrString(self, "offset", offset)
+			    || PyObject_SetAttrString(self, "text", text);
+			Py_DECREF(text);
+		    }
+		    Py_DECREF(offset);
+		}
+		Py_DECREF(lineno);
+	    }
+	    Py_DECREF(filename);
 	}
-	Py_XDECREF(filename);
-	Py_XDECREF(lineno);
-	Py_XDECREF(offset);
-	Py_XDECREF(text);
+	Py_DECREF(info);
 
 	if (status)
 	    goto finally;
@@ -791,21 +811,21 @@ SyntaxError__str__(PyObject *self, PyObject *args)
 	    if (have_filename)
 		bufsize += PyString_GET_SIZE(filename);
 
-	    buffer = PyMem_Malloc(bufsize);
+	    buffer = PyMem_MALLOC(bufsize);
 	    if (buffer != NULL) {
 		if (have_filename && have_lineno)
-		    sprintf(buffer, "%s (%s, line %ld)",
-			    PyString_AS_STRING(str),
-			    my_basename(PyString_AS_STRING(filename)),
-			    PyInt_AsLong(lineno));
+		    PyOS_snprintf(buffer, bufsize, "%s (%s, line %ld)",
+				  PyString_AS_STRING(str),
+				  my_basename(PyString_AS_STRING(filename)),
+				  PyInt_AsLong(lineno));
 		else if (have_filename)
-		    sprintf(buffer, "%s (%s)",
-			    PyString_AS_STRING(str),
-			    my_basename(PyString_AS_STRING(filename)));
+		    PyOS_snprintf(buffer, bufsize, "%s (%s)",
+				  PyString_AS_STRING(str),
+				  my_basename(PyString_AS_STRING(filename)));
 		else if (have_lineno)
-		    sprintf(buffer, "%s (line %ld)",
-			    PyString_AS_STRING(str),
-			    PyInt_AsLong(lineno));
+		    PyOS_snprintf(buffer, bufsize, "%s (line %ld)",
+				  PyString_AS_STRING(str),
+				  PyInt_AsLong(lineno));
 
 		result = PyString_FromString(buffer);
 		PyMem_FREE(buffer);
@@ -823,7 +843,7 @@ SyntaxError__str__(PyObject *self, PyObject *args)
 }
 
 
-PyMethodDef SyntaxError_methods[] = {
+static PyMethodDef SyntaxError_methods[] = {
     {"__init__", SyntaxError__init__, METH_VARARGS},
     {"__str__",  SyntaxError__str__, METH_VARARGS},
     {NULL, NULL}
@@ -831,6 +851,8 @@ PyMethodDef SyntaxError_methods[] = {
 
 
 
+/* Exception doc strings */
+
 static char
 AssertionError__doc__[] = "Assertion failed.";
 
@@ -868,6 +890,9 @@ SystemError__doc__[] = "Internal error in the Python interpreter.\n\
 Please report this to the Python maintainer, along with the traceback,\n\
 the Python version, and the hardware/OS platform and version.";
 
+static char
+ReferenceError__doc__[] = "Weak ref proxy used after referent went away.";
+
 static char
 MemoryError__doc__[] = "Out of memory.";
 
@@ -877,6 +902,28 @@ IndentationError__doc__[] = "Improper indentation.";
 static char
 TabError__doc__[] = "Improper mixture of spaces and tabs.";
 
+/* Warning category docstrings */
+
+static char
+Warning__doc__[] = "Base class for warning categories.";
+
+static char
+UserWarning__doc__[] = "Base class for warnings generated by user code.";
+
+static char
+DeprecationWarning__doc__[] =
+"Base class for warnings about deprecated features.";
+
+static char
+SyntaxWarning__doc__[] = "Base class for warnings about dubious syntax.";
+
+static char
+OverflowWarning__doc__[] = "Base class for warnings about numeric overflow.";
+
+static char
+RuntimeWarning__doc__[] =
+"Base class for warnings about dubious runtime behavior.";
+
 
 
 /* module global functions */
@@ -890,6 +937,7 @@ static PyMethodDef functions[] = {
 /* Global C API defined exceptions */
 
 PyObject *PyExc_Exception;
+PyObject *PyExc_StopIteration;
 PyObject *PyExc_StandardError;
 PyObject *PyExc_ArithmeticError;
 PyObject *PyExc_LookupError;
@@ -913,6 +961,7 @@ PyObject *PyExc_NotImplementedError;
 PyObject *PyExc_SyntaxError;
 PyObject *PyExc_IndentationError;
 PyObject *PyExc_TabError;
+PyObject *PyExc_ReferenceError;
 PyObject *PyExc_SystemError;
 PyObject *PyExc_SystemExit;
 PyObject *PyExc_UnboundLocalError;
@@ -929,6 +978,14 @@ PyObject *PyExc_WindowsError;
  */
 PyObject *PyExc_MemoryErrorInst;
 
+/* Predefined warning categories */
+PyObject *PyExc_Warning;
+PyObject *PyExc_UserWarning;
+PyObject *PyExc_DeprecationWarning;
+PyObject *PyExc_SyntaxWarning;
+PyObject *PyExc_OverflowWarning;
+PyObject *PyExc_RuntimeWarning;
+
 
 
 /* mapping between exception names and their PyObject ** */
@@ -944,6 +1001,8 @@ static struct {
   * The first three classes MUST appear in exactly this order
   */
  {"Exception", &PyExc_Exception},
+ {"StopIteration", &PyExc_StopIteration, &PyExc_Exception,
+  StopIteration__doc__},
  {"StandardError", &PyExc_StandardError, &PyExc_Exception,
   StandardError__doc__},
  {"TypeError", &PyExc_TypeError, 0, TypeError__doc__},
@@ -991,35 +1050,56 @@ static struct {
   FloatingPointError__doc__},
  {"ValueError",   &PyExc_ValueError,  0, ValueError__doc__},
  {"UnicodeError", &PyExc_UnicodeError, &PyExc_ValueError, UnicodeError__doc__},
+ {"ReferenceError",  &PyExc_ReferenceError, 0, ReferenceError__doc__},
  {"SystemError",  &PyExc_SystemError, 0, SystemError__doc__},
  {"MemoryError",  &PyExc_MemoryError, 0, MemoryError__doc__},
+ /* Warning categories */
+ {"Warning", &PyExc_Warning, &PyExc_Exception, Warning__doc__},
+ {"UserWarning", &PyExc_UserWarning, &PyExc_Warning, UserWarning__doc__},
+ {"DeprecationWarning", &PyExc_DeprecationWarning, &PyExc_Warning,
+  DeprecationWarning__doc__},
+ {"SyntaxWarning", &PyExc_SyntaxWarning, &PyExc_Warning, SyntaxWarning__doc__},
+ {"OverflowWarning", &PyExc_OverflowWarning, &PyExc_Warning,
+  OverflowWarning__doc__},
+ {"RuntimeWarning", &PyExc_RuntimeWarning, &PyExc_Warning,
+  RuntimeWarning__doc__},
  /* Sentinel */
  {NULL}
 };
 
 
 
-void
-#ifdef WIN32
-__declspec(dllexport)
-#endif /* WIN32 */
-init_exceptions(void)
+DL_EXPORT(void)
+_PyExc_Init(void)
 {
     char *modulename = "exceptions";
     int modnamesz = strlen(modulename);
     int i;
+    PyObject *me, *mydict, *bltinmod, *bdict, *doc, *args;
 
-    PyObject *me = Py_InitModule(modulename, functions);
-    PyObject *mydict = PyModule_GetDict(me);
-    PyObject *bltinmod = PyImport_ImportModule("__builtin__");
-    PyObject *bdict = PyModule_GetDict(bltinmod);
-    PyObject *doc = PyString_FromString(module__doc__);
-    PyObject *args;
+    me = Py_InitModule(modulename, functions);
+    if (me == NULL)
+	goto err;
+    mydict = PyModule_GetDict(me);
+    if (mydict == NULL)
+	goto err;
+    bltinmod = PyImport_ImportModule("__builtin__");
+    if (bltinmod == NULL)
+	goto err;
+    bdict = PyModule_GetDict(bltinmod);
+    if (bdict == NULL)
+	goto err;
+    doc = PyString_FromString(module__doc__);
+    if (doc == NULL)
+	goto err;
 
-    PyDict_SetItemString(mydict, "__doc__", doc);
+    i = PyDict_SetItemString(mydict, "__doc__", doc);
     Py_DECREF(doc);
-    if (PyErr_Occurred())
+    if (i < 0) {
+ err:
 	Py_FatalError("exceptions bootstrapping error.");
+	return;
+    }
 
     /* This is the base class of all exceptions, so make it first. */
     if (make_Exception(modulename) ||
@@ -1028,7 +1108,7 @@ init_exceptions(void)
     {
 	Py_FatalError("Base class `Exception' could not be created.");
     }
-    
+
     /* Now we can programmatically create all the remaining exceptions.
      * Remember to start the loop at 1 to skip Exceptions.
      */
@@ -1085,11 +1165,8 @@ init_exceptions(void)
 }
 
 
-void
-#ifdef WIN32
-__declspec(dllexport)
-#endif /* WIN32 */
-fini_exceptions(void)
+DL_EXPORT(void)
+_PyExc_Fini(void)
 {
     int i;
 
